/*
 *  Copyright (C) 1993  Robert Davis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of Version 2, or any later version, of 
 *  the GNU General Public License as published by the Free Software 
 *  Foundation.
 */

/* $Id: Gnuplot.h,v 1.7 1993/05/18 03:55:07 davis Exp $ */

#import <objc/Object.h>

@class GnuplotPlot, Inspector, List, Preferences;

#define	SAVE_GNUPLOT	0		/* SavePanel options	*/
#define	SAVE_EPS	1
#define	SAVE_TIFF	2

#define DOCUMENT_TYPE	"gnuplot"

@interface Gnuplot:Object
{
    NXZone *zone;

    id		infoPanel;
    id		nextstepVersionTextField;	/* auto-updated by RCS	*/
    id		versionTextField;
    id		copyingPanel;
    Inspector	*inspector;
    Preferences	*preferences;
    id		savePanelAccessory;
    id		savePanelTypeMatrix;
    id		stringSet;

    List	*docList;	/* List of documents currently open	*/
    GnuplotPlot	*currentDoc;	/* Doc with most recent main window	*/
    int		numUntitled;	/* # of Untitled docs created since launch */

    BOOL	isPoweringOff;	/* Is user powering off or logging out?	*/
}


/** Class methods for handling app-wide preferences **/

+ setConstantUpdate:(int)updateType;
+ setHalvePlot:(BOOL)condition;

- init;
- stringSet;				/* Returns the app's string set	*/


/** Target/Action **/

- showInfoPanel:sender;
- showCopyingPanel:sender;
- showPreferencesPanel:sender;
- showInspectorPane:sender;
- new:sender;
- open:sender;
- saveAll:sender;
- setSaveType:sender;
- (int)saveType;

- updateApp;			/* Updates panels that reflect current doc */


/** Interacting With Documents **/

- docDidClose:sender;
- isDocOpen: (const char *) fullPath;
- (int)numberNew;			/* Returns numUntitled		*/
- setCurrentDoc:(GnuplotPlot *)aDoc;
- (GnuplotPlot *)currentDoc;



/** Menu Auto-Update Methods **/

- (BOOL)menuItemUpdate:menuCell;
- (BOOL)validateCommand:menuCell;



/** Application delegate **/

- (int)app:sender openFile:(const char *)path type:(const char *)type;
- (BOOL)appAcceptsAnotherFile:sender;



/** 
 ** Services.  These messages are sent when a user chooses one of our 
 ** items from the services menu.
 **/

- plotData:pb userData:(const char *)userData error:(char **)errorMessage;


 @end
